/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.codepreprocessor;

/**
 *
 * @author yury
 */
public class NCharsBeforeCodePreprocessor implements CodePreprocessor {
    
    private static final int MAX_CHAR_COUNT = 1000;
    
    /* separator characters */
    private static final String NEWLINE_CHAR = "\u00ff";
    private static final String TAB_CHAR = "\uf8ff";
    private static final String SPACE_CHAR = "\uf8fe";
    
    @Override
    public String processCode(String code) {
        int codeLength = code.length();
        if (codeLength > MAX_CHAR_COUNT) {
            String relevantCode = code.substring(codeLength-MAX_CHAR_COUNT, codeLength);
            assert(relevantCode.length() == MAX_CHAR_COUNT);
            int spaceIndex = relevantCode.indexOf(" ");
            if (spaceIndex != -1) {
                relevantCode = relevantCode.substring(spaceIndex, relevantCode.length());
            }
            relevantCode = replaceSeparatorChars(relevantCode);
            return relevantCode.replace("", " ").trim();
        } else {
            String cleanedString = replaceSeparatorChars(code);
            return cleanedString.replace("", " ").trim();
        }
    }
    
    /**
     * Replaces separator characters in given string with respective Unicode symbols
     * declared on top
     * @param code
     * @return String with separators characters replaced
     */
    private String replaceSeparatorChars(String code) {
        code = code.replace(" ", SPACE_CHAR);
        code = code.replace("\t", TAB_CHAR);
        code = code.replace("\n", NEWLINE_CHAR);
        return code;
    }
    
}