/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.pythoncodecompleter;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import org.netbeans.api.editor.mimelookup.MimeRegistration;
import org.netbeans.codepreprocessor.CodePreprocessor;
import org.netbeans.codepreprocessor.NCharsBeforeCodePreprocessor;
import org.netbeans.spi.editor.completion.CompletionProvider;
import org.netbeans.spi.editor.completion.CompletionResultSet;
import org.netbeans.spi.editor.completion.CompletionTask;
import org.netbeans.spi.editor.completion.support.AsyncCompletionQuery;
import org.netbeans.spi.editor.completion.support.AsyncCompletionTask;
import org.netbeans.translationserver.TranslationRequest;
import org.netbeans.translationserver.TranslationResult;
import org.netbeans.translationserver.TranslationResultsCache;
import org.netbeans.translationserver.TranslationServerConnector;

/**
 * 
 * @author yury
 */
@MimeRegistration(mimeType = "text/x-python", service = PythonCompletionProvider.class)
public class PythonCompletionProvider implements CompletionProvider
{
    
    private final CodePreprocessor preprocessor;
    private final TranslationServerConnector serverConnector;
    private final TranslationResultsCache cache;
    
    public PythonCompletionProvider() {
        this.preprocessor = new NCharsBeforeCodePreprocessor();
        this.cache = new TranslationResultsCache();
        this.serverConnector = new TranslationServerConnector();
        this.serverConnector.setCache(this.cache);
    }

    @Override
    public CompletionTask createTask(int queryType, JTextComponent jtc) {
        if (queryType != CompletionProvider.COMPLETION_QUERY_TYPE) {
            return null;
        }
        
        return new AsyncCompletionTask(new AsyncCompletionQuery() {
            
            @Override
            protected void query(CompletionResultSet resultSet, Document doc, int caretOffset) {
                try {
                    List<TranslationResult> translationResults;
                    String relevantCode = doc.getText(0, caretOffset);
                     int dotIndex = relevantCode.lastIndexOf(".");
                    /* if we just hit the dot, make request to the server to get completion results */
                    if (doc.getText(caretOffset-1, 1).compareTo(".") == 0) {
                        String processedCode = preprocessor.processCode(relevantCode);
                        translationResults = serverConnector.translate(new TranslationRequest(processedCode));
                    } else {
                        /* if results for this query have been retrieved, use cache to get them */
                        String prefix = doc.getText(dotIndex+1, caretOffset - dotIndex - 1);
                        translationResults = cache.getStartsWith(prefix, 0);
                    }
                    for (TranslationResult translationResult: translationResults) {
                        String suggestion = translationResult.getTgt();
                        /* don't show unknown results */
                        if (suggestion.compareTo("<unk>") != 0) {
                            resultSet.addItem(new PythonCompletionItem(translationResult, dotIndex));
                        }
                    }
                } catch (BadLocationException ble) {
                    Logger.getLogger(PythonCompletionProvider.class.getName()).log(Level.SEVERE, null, ble);
                }
                resultSet.finish();
            }            
        }, jtc);
    }

    @Override
    public int getAutoQueryTypes(JTextComponent jtc, String string) {
        return 0;
    }
    
}