/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.translationserver;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author yury
 */
public class TranslationResultsCache {
    
    private final List<TranslationResult> results;
    
    public TranslationResultsCache() {
        this.results = new ArrayList<>();
    }
    
    public void cache(List<TranslationResult> resultsToCache) {
        results.clear();
        results.addAll(resultsToCache);
    }
    
    /**
     * Returns list of TranslationResult objects
     * that are filtered according with the @prefix, if set 
     * @param prefix - if not null, returns TranslationResult objects
     * that have 'tgt' starting with @prefix
     * @param limit - if set > 0, allows to set maximum limit on 
     * number of TranslationResult objects returned
     * @return filtered list of TranslationResult objects
     */
    public List<TranslationResult> getStartsWith(String prefix, int limit) {
        List<TranslationResult> filterResults = new ArrayList<>();
        for(TranslationResult result : results) {
            if (result.getTgt().startsWith(prefix)) {
                filterResults.add(result);
            }
            if (limit > 0 && filterResults.size() >= limit) {
                break;
            }
        }
        filterResults.sort(new TranslationResult.TranslationResultComparator());
        return filterResults;
    }
    
    public void clear() {
        results.clear();
    }
    
}