/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.translationserver;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import com.google.gson.Gson;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 *
 * @author yury
 */
public class TranslationServerConnector {
    
    private static final String PROPERTIES_PATH = "resources/translatorserver.properties";
    
    private final CloseableHttpClient httpClient;
    private String translatorPath;
    private TranslationResultsCache cache = null;
    
    public TranslationServerConnector() {
        this.httpClient = HttpClients.createDefault();
        this.translatorPath = "";
        try {
            Properties properties = new Properties();
            File propertiesFile = new File(PROPERTIES_PATH);
            InputStream inputStream = new FileInputStream(propertiesFile);
            if (inputStream != null) {
                properties.load(inputStream);
                String ipAddress = properties.getProperty("IP");
                try {
                    InetAddress addr = InetAddress.getByName(ipAddress);
                } catch (UnknownHostException uhe) {
                    Logger.getLogger(TranslationServerConnector.class.getName()).
                            log(Level.SEVERE, "Failed to initialize TranslationServerConnector from .properties file: invalid IP address value!", uhe);
                    System.exit(0);
                }
                String port = properties.getProperty("PORT");
                try {
                    int portAsInt = Integer.parseInt(port);
                    if (portAsInt < 0 && portAsInt >= 65536) {
                        Logger.getLogger(TranslationServerConnector.class.getName()).
                            log(Level.SEVERE, "Failed to initialize TranslationServerConnector from .properties file: invalid port value!");
                        System.exit(0);
                    }
                } catch(NumberFormatException nfe) {
                    Logger.getLogger(TranslationServerConnector.class.getName()).
                            log(Level.SEVERE, "Failed to initialize TranslationServerConnector from .properties file: invalid port value!", nfe);
                    System.exit(0);
                }
                this.translatorPath = "http://" + ipAddress + ":" + port + "/translator/translate";
            } else {
                Logger.getLogger(TranslationServerConnector.class.getName()).log(Level.SEVERE, "Failed to initialize TranslationServerConnector from .properties file!");
                System.exit(0);
            }
        } catch (IOException ix) {
            Logger.getLogger(TranslationServerConnector.class.getName()).log(Level.SEVERE, "Failed to initialize TranslationServerConnector from .properties file!", ix);
            System.exit(0);
            
        }
    }
    
    public TranslationServerConnector(InetAddress serverIP, int port) {
        if (port < 0 && port >= 65536) {
            Logger.getLogger(TranslationServerConnector.class.getName()).
                log(Level.SEVERE, "Failed to initialize TranslationServerConnector from .properties file: invalid port value!");
            System.exit(0);
        }
        this.httpClient = HttpClients.createDefault();
        String hostAddress = serverIP.getHostAddress();
        if (hostAddress == null) {
            try {
                /* set to loopback */
                hostAddress = InetAddress.getByName(null).getHostAddress();
            } catch (UnknownHostException ex) {
                Logger.getLogger(TranslationServerConnector.class.getName()).log(Level.SEVERE, "Failed to initialize TranslationServerConnector: invalid IP address value!", ex);
                System.exit(0);
            }
        }
        this.translatorPath = "http://" + hostAddress + ":" + Integer.toString(port) + "/translator/translate";
    }
    
    public void setCache(TranslationResultsCache cache) {
        this.cache = cache;
    }
    
    /**
     * Sends HTTP POST request to REST translation server that
     * translates given input using trained model and sends responses from the model back
     * @param request
     * @return List<TranslationResult>. If translation request has failed or
     * no results have been returned by the server, returns empty List.
     */
    public List<TranslationResult> translate(TranslationRequest request) {
        List<TranslationResult> translationResults = new ArrayList<>();
        HttpPost translationRequest = new HttpPost(translatorPath);
        Gson gson = new Gson();
        StringEntity parameters = new StringEntity("[" + gson.toJson(request) + "]", "UTF-8");
        translationRequest.setEntity(parameters);
        try {
            HttpResponse response = httpClient.execute(translationRequest);
            String json = EntityUtils.toString(response.getEntity(), "UTF-8");
            /* translation server returns nested array */
            TranslationResult[][] results = gson.fromJson(json, TranslationResult[][].class);
            if (results.length > 0) {
                List<TranslationResult> resultsList = Arrays.asList(results[0]);
                if (cache != null) {
                    cache.cache(resultsList);
                }
                translationResults.addAll(resultsList);
            }
            return translationResults;
        } catch (IOException ie) {
            Logger.getLogger(TranslationServerConnector.class.getName()).log(Level.SEVERE, "Error executing translation request!", ie);
        }
        try {
            httpClient.close();
        } catch (IOException ex) {
            Logger.getLogger(TranslationServerConnector.class.getName()).log(Level.SEVERE, null, ex);
        }
        return translationResults;
    }
    
}