import requests
import json
import sys
import os
import uuid
import fnmatch
from git import Repo

#Constants
SEARCH_URL = 'https://api.github.com/search/repositories?q=language:javascript&sort=stars&order=desc&per_page=100'
MAX_REPOSITORIES_NUM = 1000

#retrieves git repositories given URL and max limit on number of repositories retrieved
def retrieve_repositories(url, max_repos):
	if max_repos == 0:
		return []
	clone_urls = []
	print "Requesting {0}....".format(url)
	response = requests.get(url)
	if response.status_code == 200:
		repositories = json.loads(response.text)['items']
		print "Retrieved {0} repositories from {1}".format(len(repositories), url)
		for repository in repositories:
			if len(clone_urls) >= max_repos:
				return clone_urls
			clone_url = repository['clone_url']
			clone_urls.append(clone_url)
		if len(clone_urls) < max_repos:
			clone_urls.extend(retrieve_repositories(response.links['next']['url'], max_repos - len(clone_urls)))
			return clone_urls
		else:
			return clone_urls
	else:
		print "Failed to retrieve repositories! Status code is %d" % response.status_code
		sys.exit(0)

#extracts and returns project name from Git clone_url
def extract_project_name(url):
	git_index = url.find(".git")
	if git_index == 0:
		return "Project_" + str(uuid.uuid4())
	last_slash_index = url.rfind("/")
	if last_slash_index == 0 or last_slash_index >= git_index:
		return "Project_" + str(uuid.uuid4())
	return url[last_slash_index+ 1 : git_index]

#recursively searches and returns filenames in given path with given extension
def retrieve_files(path, extension):
	matches = []
	for root, dirnames, filenames in os.walk(path):
		for filename in fnmatch.filter(filenames, '*' + extension):
			matches.append(os.path.join(root, filename))
	return matches

def main():
	clone_urls = retrieve_repositories(SEARCH_URL, MAX_REPOSITORIES_NUM)
	print "Finished retrieving repositories! Retrieved {0} repositories.".format(len(clone_urls))

	print "Cloning repositories into /code folder..."
	for clone_url in clone_urls:
		project_name = extract_project_name(clone_url)
		#checking if 'code' folder already contains this project
		if os.path.exists('code/' + project_name):
			project_name = "Project_" + str(uuid.uuid4())
		print "Cloning {0} project from {1}...".format(project_name,clone_url)
		Repo.clone_from(clone_url, 'code/' + project_name)
	print "Finished cloning repositories!"
	print "Extracting .js files..."
	javascript_files = retrieve_files('code','.js')
	print "Script finished! Retrieved %d .js files in total!".format(len(javascript_files))


if __name__ == "__main__":
    main()
