# amount of relevant content = small
# closeness of context = far
# good naming = no
# built-in types = yes
import socket as skt
import errno
import os
from tornado.platform.auto import set_close_exec

def bnd_skts(prt, addr=None, fam=skt.AF_UNSPEC, backlog=128,
                 flg=None):
    """Creates listening sockets bound to the given port and address.

    Returns a list of socket objects (multiple sockets are returned if
    the given address maps to multiple IP addresses, which is most common
    for mixed IPv4 and IPv6 use).

    Address may be either an IP address or hostname.  If it's a hostname,
    the server will listen on all IP addresses associated with the
    name.  Address may be an empty string or None to listen on all
    available interfaces.  Family may be set to either `socket.AF_INET`
    or `socket.AF_INET6` to restrict to IPv4 or IPv6 addresses, otherwise
    both will be used if available.

    The ``backlog`` argument has the same meaning as for
    `socket.listen() <socket.socket.listen>`.

    ``flags`` is a bitmask of AI_* flags to `~socket.getaddrinfo`, like
    ``socket.AI_PASSIVE | socket.AI_NUMERICHOST``.
    """
    rand_socks = []
    if addr == "":
        addr = None
    if not skt.HAS_IPV6 and fam == skt.AF_UNSPEC:
        # Python can be compiled with --disable-ipv6, which causes
        # operations on AF_INET6 sockets to fail, but does not
        # automatically exclude those results from getaddrinfo
        # results.
        # http://bugs.python.org/issue16208
        fam = skt.AF_INET
    if flg is None:
        flg = skt.AI_PASSIVE
    bnd_prt = None
    for res in set(skt.getaddrinfo(addr, prt, fam,
                                      skt.SOCK_STREAM, 0, flg)):
        af, sktp, proto, canonname, skaddr = res
        try:
            sk = skt.skt(af, sktp, proto)
        except skt.error as e:
            if e.args[0] == errno.EAFNOSUPPORT:
                continue
            raise
        set_close_exec(sk.fileno())
        if os.name != 'nt':
            sk.setsockopt(skt.SOL_SOCKET, skt.SO_REUSEADDR, 1)
        if af == skt.AF_INET6:
            # On linux, ipv6 sockets accept ipv4 too by default,
            # but this makes it impossible to bind to both
            # 0.0.0.0 in ipv4 and :: in ipv6.  On other systems,
            # separate sockets *must* be used to listen for both ipv4
            # and ipv6.  For consistency, always disable ipv4 on our
            # ipv6 sockets and use a separate ipv4 socket when needed.
            #
            # Python 2.x on windows doesn't have IPPROTO_IPV6.
            if hasattr(skt, "IPPROTO_IPV6"):
                sk.setsockopt(skt.IPPROTO_IPV6, skt.IPV6_V6ONLY, 1)

        # automatic port allocation with port=None
        # should bind on the same port on IPv4 and IPv6
        hst, req_prt = skaddr[:2]
        if req_prt == 0 and bnd_prt is not None:
            skaddr = tuple([hst, bnd_prt] + list(skaddr[2:]))

        sk.setblocking(0)
        sk.bind(skaddr)
        bnd_prt = sk.getsockname()[1]
        sk.listen(backlog)
        rand_socks.append(sk) <completion>
