import os
import sys
import tempfile
from repositories import PyPIRepository
from logging import log
from resolver import Resolver
from exceptions import PipToolsError
from pip.req import parse_requirements

def cs_pip():
    ###
    # Parsing/collecting initial requirements
    ###
    pre=""
    rebuild=""
    allow_unsafe=True
    max_rounds=0
    pip_options = []
    generate_hashes=True

    repository = PyPIRepository()
    src_files = []

    cs = []
    for src_f in src_files:
        is_setup_file = os.path.basename(src_f) == 'setup.py'
    if is_setup_file or src_f == '-':
        # pip requires filenames and not files. Since we want to support
        # piping from stdin, we need to briefly save the input from stdin
        # to a temporary file and have pip read that.  also used for
        # reading requirements from install_requires in setup.py.
        tmp_f = tempfile.NamedTemporaryFile(mode='wt', delete=False)
        if is_setup_file:
            from distutils.core import run_setup

            dist = run_setup(src_f)
            tmp_f.write('\n'.join(dist.install_requires))
        else:
            tmp_f.write(sys.stdin.read())
        tmp_f.flush()
        cs.extend(parse_requirements(
            tmp_f.name, finder=repository.finder, session=repository.session, options=pip_options))
    else:
        cs.extend(parse_requirements(
            src_f, finder=repository.finder, session=repository.session, options=pip_options))

    # Check the given base set of constraints first
    Resolver.check_constraints(cs)

    try:
        rslvr = Resolver(cs, repository, prereleases=pre,
                         clear_caches=rebuild, allow_unsafe=allow_unsafe)
        r = rslvr.resolve(max_rounds=max_rounds)
        if generate_hashes:
            hshs = rslvr.resolve_hashes(r)
        else:
            hshs = None
    except PipToolsError as e:
        log.error(str(e))
        sys.exit(2) <completion>
